package org.deephacks.tools4j.config.internal.core;

import java.lang.reflect.Method;

import javax.enterprise.event.Observes;
import javax.enterprise.inject.spi.AfterBeanDiscovery;
import javax.enterprise.inject.spi.AnnotatedMethod;
import javax.enterprise.inject.spi.AnnotatedType;
import javax.enterprise.inject.spi.BeanManager;
import javax.enterprise.inject.spi.Extension;
import javax.enterprise.inject.spi.ProcessAnnotatedType;

import org.deephacks.tools4j.config.Config;
import org.deephacks.tools4j.config.ConfigDefault;
import org.deephacks.tools4j.config.RuntimeContext;
import org.deephacks.tools4j.config.model.Lookup;

public class ConfigCdiExtension implements Extension {
    private static final RuntimeContext ctx = Lookup.get().lookup(RuntimeContext.class);

    public void afterBeanDiscovery(@Observes AfterBeanDiscovery event, BeanManager manager) {
        event.addContext(new ConfigCdiContext());
    }

    <X> void processAnnotatedType(@Observes ProcessAnnotatedType<X> pat) {
        AnnotatedType<?> type = pat.getAnnotatedType();
        handleConfigDefault(type);
        if (!type.isAnnotationPresent(Config.class)) {
            return;
        }
        ctx.register(pat.getAnnotatedType().getJavaClass());
    }

    private void handleConfigDefault(AnnotatedType<?> type) {
        for (AnnotatedMethod<?> m : type.getMethods()) {
            ConfigDefault def = m.getAnnotation(ConfigDefault.class);
            if (def != null && m.isStatic()) {
                Method method = m.getJavaMember();
                try {
                    Object o = method.invoke(null, (Object[]) null);
                    ctx.registerDefault(o);
                } catch (Exception e) {
                    throw new RuntimeException(e);
                }
            }
        }
    }
}
